<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

if (!defined('_PS_VERSION_'))
  exit;

class VegaAutoUpdate
{
	private $module;
	private $displayName;
	private $moduleName;
	private $moduleVersion;
	
	public function __construct(Module $module)
	{
		$this->module = $module;
		$this->displayName = $module->displayName;
		$this->moduleName = $module->name;
		$this->moduleVersion = $module->version;		
		
		if (empty($this->moduleName))
			throw new Exception("Le nom du module n'a pas été spécifié");
	}
	
	public function checkUpdates()
	{
		if ($updateLookup = @file_get_contents(
				sprintf("https://polarisfiles.vega-net.net/plugins-prestashop/checkupdates.php?module=%s&module_version=%s&ps_version=%s",
						$this->moduleName,
						$this->moduleVersion,
						"1.7"
					)))
		{
			if ($updateLookup == "uptodate")
				return NULL;
			
			if (strlen($updateLookup) > 4 && substr($updateLookup, 0, 4) == 'http')
				return $updateLookup;

			throw new Exception("Une erreur s'est produite sur le serveur de mises à jour : ".$updateLookup);
		}
		else		
			throw new Exception("impossible de contacter le serveur de mises à jour");
	}
	
	public function autoUpdate()
	{
		$optName = "VEGA_AUTO_UPDATER_LAST_CHECK_FOR_".$this->moduleName;
		$lastCheck = Configuration::Get($optName);
		if (!$lastCheck || (time() - $lastCheck) > (24 * 3600))
		{
			try
			{
				$this->doUpdates();				
			}
			catch (Exception $ex)
			{
				$errors = array();
				while ($ex != null)
				{
					$msg = $ex->getMessage();
					
					if (true)
						$msg .= " @ ".basename($ex->getFile())." Lg. ".$ex->getLine();
					
					$errors[] = $msg;
					
					$ex = $ex->getPrevious();
				}
				
				$error = join(" ➔ ", $errors);				
				
				PolarisLink::Log(LOG_ERR, "Erreur lors de la mise à jour : " . $error, 1);
			}
			Configuration::updateValue($optName, time());
		}
	}
	
	private static function deleteDir($dirPath) 
	{
		if( !file_exists($dirPath))
			return;	// Le répertoire n'existe pas, fin !
			
		if (! is_dir($dirPath)) {
			throw new InvalidArgumentException("$dirPath must be a directory");
		}
		if (substr($dirPath, strlen($dirPath) - 1, 1) != '/') {
			$dirPath .= '/';
		}
		$files = glob($dirPath . '*', GLOB_MARK);
		foreach ($files as $file) {
			if (is_dir($file)) {
				self::deleteDir($file);
			} else {
				unlink($file);
			}
		}
		rmdir($dirPath);
	}

	public function doUpgrade($fromVersion)
	{
		// Surtout plus de cache !!!
		Db::getInstance()->disableCache();
		
		$module = null;
		foreach (Module::getModulesOnDisk() as $m)
		{
			if ($this->module->name == $m->name)
			{
				$module = $m;
				break;
			}
		}

		if (!empty($module))
		{
			if (Module::initUpgradeModule($module))
			{
				$upgradeResults = $this->module->runUpgradeModule();
				if ($upgradeResults["success"])
				{
					$this->module->registerHooks();
					PolarisLink::Log(LOG_INFO, "Votre module a été mis à jour avec succès (%s → %s)", 1, [$fromVersion, $this->moduleVersion]);
				}
				else
				{
					throw new Exception("Un problème a été rencontré lors de la mise à jour du module, merci de terminer l'opération dans le gestionnaire de modules !");
				}
			}			
		}
		else
			throw new Exception("Un problème a été rencontré lors de la mise à jour du module, merci de terminer l'opération dans le gestionnaire de modules !");
	}
	
	public function doUpdates()
	{		
		$update = $this->checkUpdates();
		if (empty($update))
			return false;

		$toVersion = "?";
		
		if (($idx = stripos($update, "download=")) !== FALSE)
			$toVersion = substr($update, $idx + 9);
		
		clearstatcache();
		
		// Téléchargement de la mise à jour
		$updateData = @file_get_contents($update);
		if (!$updateData)
			throw new Exception("Impossible de télécharger la mise à jour ".$update);
		
		// Ok, préparation de la mise à jour ...
		$tmpBackupModule = _PS_ROOT_DIR_."/polarislink/updates/".$this->moduleName."-backup";
		$tmpDir = _PS_ROOT_DIR_."/polarislink/updates/".$this->moduleName."-new";
		
		// Nettoyage
		if (file_exists($tmpBackupModule))
			self::deleteDir($tmpBackupModule);
		if (file_exists($tmpDir))
			self::deleteDir($tmpDir);
		
		if (!mkdir($tmpDir, 0777, true))
			throw new Exception(sprintf("Impossible de créer le répertoire temporaire de mise à jour (%s)", $tmpDir));
		
		if (!file_put_contents($tmpDir."/archive.tar.gz", $updateData))
			throw new Exception(sprintf("Impossible de sauver les données de mise à jour (%s)", $tmpDir."/archive.tar.gz"));
		
		$phar = new PharData($tmpDir."/archive.tar.gz");
		$phar->extractTo($tmpDir);
		
		if (!file_exists($tmpDir."/".$this->moduleName))
			throw new Exception("Contenu de la mise à jour introuvable!");
		
		// Remplacement module actuel ...
		$modulePath = _PS_ROOT_DIR_."/modules/".$this->moduleName;		

		self::deleteDir($tmpBackupModule);
		if (!rename($modulePath, $tmpBackupModule))
			throw new Exception(sprintf("Impossible de sauvegarder l'état du module actuel (vérifiez les droits en écriture sur le répertoire %s", $tmpBackupModule));
		
		if (!rename($tmpDir."/".$this->moduleName, $modulePath))
		{
			if (!rename($tmpBackupModule, $modulePath))
				throw new Exception("Impossible de restaurer l'état du module après une tentative de mise à jour échouée");
			throw new Exception("Impossible de remplacer le répertoire du module");
		}
		
		// Nettoyage
		self::deleteDir($tmpBackupModule);
		self::deleteDir($tmpDir);
		
		return true;
	}
}